/*
 * Copyright (c) 2014 Max Gttlicher
 *
 * This software is provided 'as-is', without any express or implied warranty. In no event will the authors be held liable for any damages arising from the use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose, including commercial applications, and to alter it and redistribute it freely, subject to the following restrictions:
 * 
 * The origin of this software must not be misrepresented; you must not claim that you wrote the original software. If you use this software in a product, an acknowledgment in the product documentation would be appreciated but is not required.
 *     Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
 * This notice may not be removed or altered from any source distribution.
 */

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;


public class Configfile {
	private File config;

	public Configfile(File config) throws IOException {
		this.config = config;
		if(!config.canRead() || !config.isFile()) {
			throw new IOException("Can't read " + config);
		}
	}

	public String readKey(String section, String key) {
		BufferedReader br = null;
		try {
			br = new BufferedReader(new FileReader(config));
			String line, lc = section.toLowerCase();
			while((line = br.readLine().toLowerCase()) != null) {
				int start = 0;
				
				if(line.length() > 1) {
					while(line.charAt(start) == ' ' && start < line.length() - 2) start++;
					if(line.charAt(start) == '[') {
						if(line.indexOf(lc) != -1) {
							break;
						}
					}
				}
			}

			while((line = br.readLine()) != null) {
				int start = 0, end = line.length() - 1, separator, eol;
				String k, v;

				//Anythis behind #is comment and not interesting at all
				eol = line.indexOf('#');
				if(eol == -1) {
					eol = line.length();
				}
				if(eol == 0) {
					continue;
				}

				separator = line.indexOf('=');

				end = separator;

				//Remove whitespaces
				while(line.charAt(start) == ' ' && start < line.length() - 1) {
					++start;
				}

				if(line.charAt(start) == '[') {

					break;
				}

				if(separator == -1) {
					continue;
				}

				while(line.charAt(end - 1) == ' ') {
					--end;
				}

				if(end > eol) {
					continue;
				}

				k = line.substring(start, end);



				if (k.toLowerCase().equals(key.toLowerCase()) && k.charAt(0) != '#') {
					start = separator + 1;
					end = eol;
					//Remove whitespaces
					while (line.charAt(start) == ' ' && start < eol - 1) {
						++start;
					}
					while (line.charAt(end - 1) == ' ' && start > 1) {
						--end;
					}
					if (end > eol) {
						continue;
					}
					v = line.substring(start, end);

					br.close();

					return v;
				}
			}
			br.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				br.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}



		return null;
	}

	public HashMap<String, String> readSection(String name) {
		BufferedReader br = null;
		try {
			br = new BufferedReader(new FileReader(config));

			HashMap<String, String> entries = new HashMap<>();
			String line, lc = name.toLowerCase();
			while((line = br.readLine().toLowerCase()) != null) {
				int start = 0;
				while(line.charAt(start) == ' ') start++;
				if(line.charAt(start) == '[') {
					if(line.indexOf(lc) != -1) {
						break;
					}
				}
			}
			while((line = br.readLine()) != null) {
				int start = 0, end = line.length() - 1, separator, eol;
				String k, v;

				//Anythis behind #is comment and not interesting at all
				eol = line.indexOf('#');
				if(eol == -1) {
					eol = line.length();
				}
				if(eol == 0) {
					continue;
				}

				separator = line.indexOf('=');

				end = separator;

				//Remove whitespaces
				while(line.charAt(start) == ' ') {
					++start;
				}

				if(line.charAt(start) == '[') {

					break;
				}

				if(separator == -1) {
					continue;
				}

				while(line.charAt(end - 1) == ' ') {
					--end;
				}

				if(end > eol) {
					continue;
				}

				k = line.substring(start, end);

				start = separator + 1;
				end = eol;

				//Remove whitespaces
				while(line.charAt(start) == ' ') {
					++start;
				}

				while(line.charAt(end - 1) == ' ') {
					--end;
				}

				if(end > eol) {
					continue;
				}

				v = line.substring(start, end);
				if(k.charAt(0) != '#')
					entries.put(k, v);
			}
			br.close();
			return entries;

		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} finally {
			try {
				br.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		return null;
	}
}
