/*
 * Copyright (c) 2014 Max Gttlicher
 *
 * This software is provided 'as-is', without any express or implied warranty. In no event will the authors be held liable for any damages arising from the use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose, including commercial applications, and to alter it and redistribute it freely, subject to the following restrictions:
 * 
 * The origin of this software must not be misrepresented; you must not claim that you wrote the original software. If you use this software in a product, an acknowledgment in the product documentation would be appreciated but is not required.
 *     Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
 * This notice may not be removed or altered from any source distribution.
 */

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.EOFException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.TreeMap;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import javax.imageio.ImageIO;

import tiimage.TIImage;


public class Converter {
	Configfile cfg;
	
	File pack;
	
	public static String TEXTURE_HEADER = "IDTexture";
	private static int OPERATIONS_PER_VAR = 15;
	
	public Converter(File pack, File config) {
		this.pack = pack;
		try {
			cfg = new Configfile(config);
		} catch (IOException e) {
			e.printStackTrace();
			return;
		}
	}
	
	public void writeToFile(File template, File output, String version, String author, String compat, String description) {
		String outfile = "";
		
		try {
			BufferedReader br = new BufferedReader(new FileReader(template));
			String line;
			while((line = br.readLine()) != null) {
				outfile = outfile.concat(line + "\n");
				
			}
			br.close();
			List<String> vars = getVars();
			
			String content = "";
			for(int i=0;i<vars.size();++i) {
				content = String.format("%scontent[%d] = %s\n", content, i, vars.get(i));
			}
			
			
			String filename = output.getName();
			filename = filename.substring(0, filename.lastIndexOf('.'));
			
			outfile = String.format(outfile, content, version, author, compat, description.concat(" - port: MC2D TPC"), filename);
			
			
			
			BufferedWriter bw = new BufferedWriter(new FileWriter(output));
			bw.write(outfile);
			bw.close();
			
		} catch (FileNotFoundException e) {
			
			e.printStackTrace();
		} catch (IOException e) {
			
			e.printStackTrace();
		}
		
	}
	
	public List<String> getVars() {
		List<String> operations = convertTexturesToLua();
		List<String> vars = new ArrayList<>();
		
		String var = "";
		int added = 0;
		for(int i = 0;i < operations.size();++i) {
			String op = operations.get(i).replaceAll("\\\\", "\\\\\\\\");
			var = var.concat(op);
			var = var.concat("\\n");
			if(added == OPERATIONS_PER_VAR || i == operations.size() - 1) {
				vars.add(String.format("\"%s\"", var));
				var = "";
				added = 0;
			}
			++added;
		}
		
		return vars;
	}
	
	public List<String> convertTexturesToLua() {
		ArrayList<String> lua = new ArrayList<>();
		
		Map<String, String> textureNames;
		Map<String, BufferedImage> textures = new TreeMap<String, BufferedImage>();
		
		List<String> stair_left = getTokens(cfg.readKey("stair", "left"), ",");
		List<String> stair_right = getTokens(cfg.readKey("stair", "right"), ",");
		List<String> chest = getTokens(cfg.readKey("chest", "id"), ",");
		List<String> newtag = getTokens(cfg.readKey("NewTexture", "id"), ",");
		textureNames = cfg.readSection(TEXTURE_HEADER);
		
		
		
		for(String key : textureNames.keySet()) {
			String path = textureNames.get(key);
			if(Character.isDigit(path.charAt(0))) {
				try {
					path = textureNames.get(path);
				} catch (Exception e) {
					//Image can't be loaded, continue
					e.printStackTrace();
					continue;
				}
			} 
			BufferedImage v = findImage(path);
			
			if(v == null) {
				continue;
			}
			
			//Insert image operations here...
			if(stair_left.contains(key)) {
				v = stairModifier(v, false);
			} else if(stair_right.contains(key)) {
				v = stairModifier(v, true);
			}
			if(chest.contains(key)) {
				v = chestModifier(v);
			}
			
			textures.put(key, v);
			
		}
		
		for(String key : textures.keySet()) {
			boolean isNew = newtag.contains(key);
			if(isNew) {
				System.out.println(key);
			}
			TIImage img = new TIImage(textures.get(key).getSubimage(0, 0, 16, 16));
			lua.add(String.format("IDTexture[%s] = %s\'%s\'%s", key, isNew ? "image.new(" : "", img.getString(), isNew ? ")" : ""));
		}
		return lua;
	}
	
	private BufferedImage stairModifier(BufferedImage mod, boolean right) {
		int x = right ? 8 : 0;
		BufferedImage img = new BufferedImage(mod.getWidth(), mod.getHeight(), BufferedImage.TYPE_INT_ARGB);
		Graphics2D g = img.createGraphics();
		g.drawImage(mod, 0, 0, 16, 16, 0, 0, 16, 16, null);
		g.setBackground(new Color(0, 0, 0, 0));
		g.clearRect(x, 0, 8, 8);
		g.dispose();
		return img;
	}
	
	private BufferedImage chestModifier(BufferedImage src) {
		
		BufferedImage chest = new BufferedImage(16, 16, BufferedImage.TYPE_INT_ARGB);
		
		Graphics2D g = chest.createGraphics();
		
		g.drawImage(src, 1, 6, 15, 16, 14, 33, 28, 43, null);
		g.drawImage(src, 1, 2, 15, 7, 14, 14, 28, 19, null);
		g.dispose();
		return chest;
	}
	
	private List<String> getTokens(String src, String sep) {
		ArrayList<String> tokens = new ArrayList<>();
		StringTokenizer tokenizer = new StringTokenizer(src, sep);
		
		while(tokenizer.hasMoreElements()) {
			tokens.add(tokenizer.nextToken());
		}
		
		return tokens;
	}
	
	
	private BufferedImage findImage(String entry) {
		BufferedImage img = null;
		try {
			img = ImageIO.read(getZipInputStream(pack, entry));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (EOFException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return img;
	}
	
	public InputStream getZipInputStream(File zip, String entry) throws FileNotFoundException, EOFException {
		ZipInputStream zin = new ZipInputStream(new FileInputStream(zip));
		ZipEntry e;
		try {
			while((e = zin.getNextEntry())!=null) {
				if(e.getName().toLowerCase().equals(entry.toLowerCase())) {
					return zin;
				}
			}
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		throw new EOFException("Can't find " + entry);
	}
	
	
	
	public Configfile getConfig() {
		return cfg;
	}
	
	public void setConfig(Configfile cfg) {
		this.cfg = cfg;
	}

}
