/*
 * Copyright (c) 2014 Max Gttlicher
 *
 * This software is provided 'as-is', without any express or implied warranty. In no event will the authors be held liable for any damages arising from the use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose, including commercial applications, and to alter it and redistribute it freely, subject to the following restrictions:
 * 
 * The origin of this software must not be misrepresented; you must not claim that you wrote the original software. If you use this software in a product, an acknowledgment in the product documentation would be appreciated but is not required.
 *     Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
 * This notice may not be removed or altered from any source distribution.
 */

import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.UIManager;
import javax.swing.UIManager.LookAndFeelInfo;
import javax.swing.border.BevelBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.JLabel;
import javax.swing.JTextPane;
import javax.swing.JScrollPane;


public class ConverterGUI extends JFrame {

	/**
	 * 
	 */
	private static final long serialVersionUID = -199732063678418872L;
	
	private JPanel contentPane;
	private JTextField txtResourcePack;
	private JTextField txtTemplate;

	private File resourcePack;
	private File template;
	private File config;
	
	private JFileChooser jf = new JFileChooser(new File("."));

	private JButton btnGenerate;
	private JTextField txtConfig;
	private JButton btnLoadConfig;
	private JTextField txtVersion;
	private JTextField txtAuthor;
	private JTextField txtCompat;

	private JTextPane txtpnDescription;
	
	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					ConverterGUI frame = new ConverterGUI();
					frame.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	private void refreshLabels() {
		
		if(resourcePack != null) {
			
			if(!resourcePack.isFile()) {
				resourcePack = null;
			} else {
				txtResourcePack.setText(resourcePack.getName());
			}
		}
		
		if(template != null) {
			
			if(!template.isFile()) {
				template = null;
			} else {
				txtTemplate.setText(template.getName());
			}
		}
		
		if(config != null) {
			
			if(!config.isFile()) {
				config = null;
			} else {
				txtConfig.setText(config.getName());
			}
		}

		btnGenerate.setEnabled(template != null && resourcePack != null && config != null);

	}
	
	/**
	 * Create the frame.
	 */
	public ConverterGUI() {
		
		try {
		    for (LookAndFeelInfo info : UIManager.getInstalledLookAndFeels()) {
		        if ("Nimbus".equals(info.getName())) {
		            UIManager.setLookAndFeel(info.getClassName());
		            break;
		        }
		    }
		} catch (Exception e) {
		    // If Nimbus is not available, you can set the GUI to another look and feel.
		}
		
		setTitle("MC2D Texturepack Converter");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setBounds(100, 100, 390, 393);
		contentPane = new JPanel();
		contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		setContentPane(contentPane);
		
		JPanel panel = new JPanel();
		panel.setBorder(new BevelBorder(BevelBorder.LOWERED, null, null, null, null));
		GroupLayout gl_contentPane = new GroupLayout(contentPane);
		gl_contentPane.setHorizontalGroup(
			gl_contentPane.createParallelGroup(Alignment.LEADING)
				.addComponent(panel, Alignment.TRAILING, GroupLayout.DEFAULT_SIZE, 424, Short.MAX_VALUE)
		);
		gl_contentPane.setVerticalGroup(
			gl_contentPane.createParallelGroup(Alignment.LEADING)
				.addComponent(panel, GroupLayout.DEFAULT_SIZE, 252, Short.MAX_VALUE)
		);
		
		txtResourcePack = new JTextField();
		txtResourcePack.setEnabled(false);
		txtResourcePack.setEditable(false);
		txtResourcePack.setColumns(10);
		
		JButton btnLoadResourcePack = new JButton("Load Resource Pack");
		btnLoadResourcePack.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				jf.setDialogTitle("Open Resource Pack");
				int opt = jf.showOpenDialog(ConverterGUI.this);
				if(opt == JFileChooser.APPROVE_OPTION) {
					resourcePack = jf.getSelectedFile();
				}
				
				refreshLabels();
			}
		});
		
		txtTemplate = new JTextField();
		txtTemplate.setEnabled(false);
		txtTemplate.setEditable(false);
		txtTemplate.setColumns(10);
		
		JButton btnSaveLuaScript = new JButton("Load Template");
		btnSaveLuaScript.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				jf.setDialogTitle("Open Script Template File");
				int opt = jf.showOpenDialog(ConverterGUI.this);
				if(opt == JFileChooser.APPROVE_OPTION) {
					template = jf.getSelectedFile();
				}
				
				refreshLabels();
			}
		});
		
		btnLoadConfig = new JButton("Load Config");
		btnLoadConfig.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				jf.setDialogTitle("Open Config File");
				int opt = jf.showOpenDialog(ConverterGUI.this);
				if(opt == JFileChooser.APPROVE_OPTION) {
					config = jf.getSelectedFile();
				}
				
				refreshLabels();
			}
		});
		
		btnGenerate = new JButton("Generate Lua Script");
		btnGenerate.setToolTipText("Generates a Lua script that loads the textures into as library. Use Luna to create a working .tns file from the script.");
		btnGenerate.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				jf.setDialogTitle("Save Lua Script");
				File s = jf.getSelectedFile();
				if(s != null) {
					String name = s.getPath();
					name = name.substring(0, name.lastIndexOf('.')).concat(".lua");
					File n = new File(name);
					jf.setSelectedFile(n);
				}
				int opt = jf.showSaveDialog(ConverterGUI.this);
				if(opt == JFileChooser.APPROVE_OPTION) {
					File script = jf.getSelectedFile();
					Converter convert = new Converter(resourcePack, config);
					String error = null;
					try {
						convert.writeToFile(template, script, txtVersion.getText(), txtAuthor.getText(), txtCompat.getText(), txtpnDescription.getText());
					} catch (Exception e1) {
						e1.printStackTrace();
						error = e1.getMessage();
					}
					if(error == null) {
						JOptionPane.showMessageDialog(ConverterGUI.this, "Conversion completed", "Success", JOptionPane.INFORMATION_MESSAGE);
					} else {
						JOptionPane.showMessageDialog(ConverterGUI.this, "Conversion failed: ".concat(error), "Error", JOptionPane.ERROR_MESSAGE);
					}
				}
			}
		});
		btnGenerate.setEnabled(false);
		
		txtConfig = new JTextField();
		txtConfig.setEnabled(false);
		txtConfig.setEditable(false);
		txtConfig.setColumns(10);
		
		txtVersion = new JTextField();
		txtVersion.setText("1.0");
		txtVersion.setColumns(10);
		
		JLabel lblVersion = new JLabel("Version");
		
		JLabel lblAuthor = new JLabel("Author");
		
		txtAuthor = new JTextField();
		txtAuthor.setToolTipText("This should be the name of the original author of the pack");
		txtAuthor.setColumns(10);
		
		JLabel lblCompatible = new JLabel("Compatible");
		
		txtCompat = new JTextField();
		txtCompat.setText("all");
		txtCompat.setColumns(10);
		
		JScrollPane scrollPane = new JScrollPane();
		
		GroupLayout gl_panel = new GroupLayout(panel);
		gl_panel.setHorizontalGroup(
			gl_panel.createParallelGroup(Alignment.TRAILING)
				.addGroup(gl_panel.createSequentialGroup()
					.addGroup(gl_panel.createParallelGroup(Alignment.LEADING)
						.addGroup(gl_panel.createSequentialGroup()
							.addContainerGap()
							.addComponent(btnGenerate, GroupLayout.DEFAULT_SIZE, 348, Short.MAX_VALUE))
						.addGroup(gl_panel.createSequentialGroup()
							.addGap(9)
							.addGroup(gl_panel.createParallelGroup(Alignment.TRAILING)
								.addComponent(txtResourcePack, GroupLayout.DEFAULT_SIZE, 176, Short.MAX_VALUE)
								.addComponent(txtTemplate, GroupLayout.DEFAULT_SIZE, 176, Short.MAX_VALUE))
							.addPreferredGap(ComponentPlacement.RELATED)
							.addGroup(gl_panel.createParallelGroup(Alignment.LEADING, false)
								.addComponent(btnLoadResourcePack, Alignment.TRAILING, GroupLayout.PREFERRED_SIZE, 163, GroupLayout.PREFERRED_SIZE)
								.addComponent(btnSaveLuaScript, GroupLayout.PREFERRED_SIZE, 163, GroupLayout.PREFERRED_SIZE)))
						.addGroup(Alignment.TRAILING, gl_panel.createSequentialGroup()
							.addGroup(gl_panel.createParallelGroup(Alignment.TRAILING)
								.addGroup(Alignment.LEADING, gl_panel.createSequentialGroup()
									.addContainerGap()
									.addComponent(lblCompatible)
									.addPreferredGap(ComponentPlacement.RELATED)
									.addComponent(txtCompat, GroupLayout.DEFAULT_SIZE, 109, Short.MAX_VALUE))
								.addGroup(Alignment.LEADING, gl_panel.createSequentialGroup()
									.addGap(9)
									.addComponent(txtConfig, GroupLayout.DEFAULT_SIZE, 176, Short.MAX_VALUE))
								.addGroup(Alignment.LEADING, gl_panel.createSequentialGroup()
									.addContainerGap()
									.addGroup(gl_panel.createParallelGroup(Alignment.LEADING)
										.addComponent(lblVersion)
										.addComponent(lblAuthor))
									.addPreferredGap(ComponentPlacement.RELATED)
									.addGroup(gl_panel.createParallelGroup(Alignment.LEADING)
										.addComponent(txtAuthor, GroupLayout.DEFAULT_SIZE, 131, Short.MAX_VALUE)
										.addComponent(txtVersion, GroupLayout.DEFAULT_SIZE, 131, Short.MAX_VALUE))))
							.addPreferredGap(ComponentPlacement.RELATED)
							.addComponent(btnLoadConfig, GroupLayout.PREFERRED_SIZE, 163, GroupLayout.PREFERRED_SIZE))
						.addGroup(gl_panel.createSequentialGroup()
							.addContainerGap()
							.addComponent(scrollPane, GroupLayout.DEFAULT_SIZE, 348, Short.MAX_VALUE)))
					.addContainerGap())
		);
		gl_panel.setVerticalGroup(
			gl_panel.createParallelGroup(Alignment.LEADING)
				.addGroup(gl_panel.createSequentialGroup()
					.addContainerGap()
					.addGroup(gl_panel.createParallelGroup(Alignment.BASELINE)
						.addComponent(btnLoadResourcePack)
						.addComponent(txtResourcePack, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE))
					.addPreferredGap(ComponentPlacement.RELATED)
					.addGroup(gl_panel.createParallelGroup(Alignment.BASELINE)
						.addComponent(txtTemplate, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
						.addComponent(btnSaveLuaScript))
					.addPreferredGap(ComponentPlacement.RELATED)
					.addGroup(gl_panel.createParallelGroup(Alignment.BASELINE)
						.addComponent(txtConfig, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
						.addComponent(btnLoadConfig))
					.addPreferredGap(ComponentPlacement.RELATED)
					.addGroup(gl_panel.createParallelGroup(Alignment.BASELINE)
						.addComponent(lblVersion)
						.addComponent(txtVersion, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE))
					.addPreferredGap(ComponentPlacement.RELATED)
					.addGroup(gl_panel.createParallelGroup(Alignment.BASELINE)
						.addComponent(lblAuthor)
						.addComponent(txtAuthor, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE))
					.addPreferredGap(ComponentPlacement.RELATED)
					.addGroup(gl_panel.createParallelGroup(Alignment.BASELINE)
						.addComponent(lblCompatible)
						.addComponent(txtCompat, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE))
					.addPreferredGap(ComponentPlacement.RELATED)
					.addComponent(scrollPane, GroupLayout.DEFAULT_SIZE, 59, Short.MAX_VALUE)
					.addPreferredGap(ComponentPlacement.RELATED)
					.addComponent(btnGenerate, GroupLayout.PREFERRED_SIZE, 60, GroupLayout.PREFERRED_SIZE)
					.addContainerGap())
		);
		
		txtpnDescription = new JTextPane();
		scrollPane.setViewportView(txtpnDescription);
		panel.setLayout(gl_panel);
		contentPane.setLayout(gl_contentPane);
		
		//Try to load default files
		
		template = new File("template.lua");
		if(!template.isFile()) {
			template = null;
		}
		
		config = new File("config.txt");
		if(!config.isFile()) {
			config = null;
		}
		
		refreshLabels();
		
	}
}
